// CoJak: The CJK Oracular Assistant
// Copyright (C) 2003 by Tal Liron
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA

package gnu.cojak;

import org.apache.lucene.index.*;
import org.apache.lucene.analysis.*;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.analysis.cn.ChineseAnalyzer;
import org.apache.lucene.document.*;
import org.apache.lucene.search.*;
import org.apache.lucene.queryParser.*;
import org.apache.lucene.store.*;

import java.io.*;
import java.util.*;

public class Index
{
	//
	// Main
	//
	
	public static void main( String[] arguments ) throws Exception
	{
		Index index = new Index( true );
		index.indexChineseCharacters( true );
		index.indexChineseWords( false );
		index.indexJapaneseCharacters( false );
		index.indexJapaneseWords( false );
		index.indexKoreanWords( false );
		index.optimize();
	}
	
	//
	// Construction
	//
	
	public Index() throws IOException
	{
		this( false );
	}
	
	public Index( boolean create ) throws IOException
	{
		this( FSDirectory.getDirectory( "index", create ) );
	}
	
	public Index( Directory directory )
	{
		this.directory = directory;
	}
	
	//
	// Operations
	//
	
	public Hits search( String queryString, List fieldList ) throws IOException, ParseException
	{
		String[] fields = new String[ fieldList.size() ];
		fieldList.toArray( fields );
		Searcher searcher = new IndexSearcher( directory );
		Query query = MultiFieldQueryParser.parse( queryString, fields, analyzer );
		return searcher.search( query );
	}
	
	public void optimize() throws IOException
	{
		IndexWriter indexWriter = null;
		try
		{
			indexWriter = new IndexWriter( directory, analyzer, false );
			System.out.println( "Optimizing index..." );
			indexWriter.optimize();
		}
		finally
		{
			if( indexWriter != null )
			{
				indexWriter.close();
			}
		}
	}
	
	public void index( IndexReader reader, boolean create ) throws IOException
	{
		IndexWriter indexWriter = null;
		try
		{
			indexWriter = new IndexWriter( directory, analyzer, create );
			int count = 0;
			System.out.println( "Indexing..." );
			for( int i = 0; i < reader.numDocs(); i++ )
			{
				indexWriter.addDocument( reader.document( i ) );
				if( (count++) % 100 == 0 )
				{
					System.out.print( "#" );
				}
			}
			System.out.println();
		}
		finally
		{
			if( indexWriter != null )
			{
				indexWriter.close();
			}
		}
	}
	
	public void indexChineseCharacters( boolean create ) throws IOException
	{
		IndexWriter indexWriter = null;
		try
		{
			indexWriter = new IndexWriter( directory, analyzer, create );
			ChineseCharacter character;
			Document document;
			int count = 0;
			System.out.println( "Indexing ccdict..." );
			for( ChineseCharacterIterator i = new ChineseCharacterIterator( new File( "data/ccdict.txt" ) ); i.hasNext(); )
			{
				character = (ChineseCharacter) i.next();
				document = new Document();
				document.add( Field.Text( "chineseHanzi", "" + character.chineseHanzi ) );
				if( character.kangxiRadical != null )
				{
					document.add( Field.Text( "kangxiRadical", character.kangxiRadical ) );
					document.add( Field.Text( "kangxi", character.kangxiRadical + "." + character.kangxiIndex ) );
				}
				if( character.kangxiAltRadical != null )
				{
					document.add( Field.Text( "kangxiRadical", character.kangxiAltRadical ) );
					document.add( Field.Text( "kangxi", character.kangxiAltRadical + "." + character.kangxiAltIndex ) );
				}
				if( character.strokes != 0 ) document.add( Field.Text( "strokes", "" + character.strokes ) );
				if( character.cangjie != null ) document.add( Field.Text( "cangjie", character.cangjie ) );
				if( character.fourCorner != null ) document.add( Field.Text( "fourCorner", character.fourCorner ) );
				if( character.hakkaMacIver != null ) document.add( Field.Text( "hakkaMacIver", character.hakkaMacIver ) );
				if( character.hakkaRey != null ) document.add( Field.Text( "hakkaRey", character.hakkaRey ) );
				if( character.hakkaHagfaPinyim != null ) document.add( Field.Text( "hakkaHagfaPinyim", character.hakkaHagfaPinyim ) );
				if( character.hakkaSiyan != null ) document.add( Field.Text( "hakkaSiyan", character.hakkaSiyan ) );
				if( character.hakkaHailu != null ) document.add( Field.Text( "hakkaHailu", character.hakkaHailu ) );
				if( character.hakkaMeixian != null ) document.add( Field.Text( "hakkaMeixian", character.hakkaMeixian ) );
				if( character.cantoneseJyutpin != null ) document.add( Field.Text( "cantoneseJyutpin", character.cantoneseJyutpin ) );
				if( character.mandarinPinyin != null ) document.add( Field.Text( "mandarinPinyin", character.mandarinPinyin ) );
				if( character.mandarinTongyong != null ) document.add( Field.Text( "mandarinTongyong", character.mandarinTongyong ) );
				if( character.english != null ) document.add( Field.Text( "english", character.english ) );
				indexWriter.addDocument( document );
				if( (++count) % 100 == 0 )
				{
					System.out.print( "#" );
				}
			}
			System.out.println();
		}
		finally
		{
			if( indexWriter != null )
			{
				indexWriter.close();
			}
		}
	}
	
	public void indexChineseWords( boolean create ) throws IOException
	{
		IndexWriter indexWriter = null;
		try
		{
			indexWriter = new IndexWriter( directory, analyzer, create );
			ChineseWord word;
			Document document;
			int count = 0;
			System.out.println( "Indexing cedict..." );
			for( ChineseWordIterator i = new ChineseWordIterator( new File( "data/cedict.b5" ) ); i.hasNext(); )
			{
				word = (ChineseWord) i.next();
				document = new Document();
				document.add( Field.Text( "chineseHanzi", word.chineseHanzi ) );
				document.add( Field.Text( "mandarinPinyin", word.mandarinPinyin ) );
				document.add( Field.Text( "english", word.english ) );
				indexWriter.addDocument( document );
				if( (++count) % 100 == 0 )
				{
					System.out.print( "#" );
				}
			}
			System.out.println();
		}
		finally
		{
			if( indexWriter != null )
			{
				indexWriter.close();
			}
		}
	}
	
	public void indexJapaneseCharacters( boolean create ) throws IOException
	{
		IndexWriter indexWriter = null;
		try
		{
			indexWriter = new IndexWriter( directory, analyzer, create );
			JapaneseCharacter character;
			Document document;
			int count = 0;
			TermDocs docs;
			System.out.println( "Indexing kanjidic..." );
			for( JapaneseCharacterIterator i = new JapaneseCharacterIterator( new File( "data/kanjidic" ) ); i.hasNext(); )
			{
				character = (JapaneseCharacter) i.next();
				document = new Document();
				document.add( Field.Text( "japaneseKanji", "" + character.japaneseKanji ) );
				if( character.strokes != 0 ) document.add( Field.Text( "strokes", "" + character.strokes ) );
				if( character.fourCorner != null ) document.add( Field.Text( "fourCorner", character.fourCorner ) );
				if( character.japaneseKana != null ) document.add( Field.Text( "japaneseKana", character.japaneseKana ) );
				if( character.mandarinPinyin != null ) document.add( Field.Text( "mandarinPinyin", character.mandarinPinyin ) );
				if( character.koreanRomanization != null ) document.add( Field.Text( "koreanRomanization", character.koreanRomanization ) );
				if( character.english != null ) document.add( Field.Text( "english", character.english ) );
				indexWriter.addDocument( document );
				if( (++count) % 100 == 0 )
				{
					System.out.print( "#" );
				}
			}
			System.out.println();
		}
		finally
		{
			if( indexWriter != null )
			{
				indexWriter.close();
			}
		}
	}
	
	public void indexJapaneseWords( boolean create ) throws IOException
	{
		IndexWriter indexWriter = null;
		try
		{
			indexWriter = new IndexWriter( directory, analyzer, create );
			JapaneseWord word;
			Document document;
			int count = 0;
			TermDocs docs;
			System.out.println( "Indexing edict..." );
			for( JapaneseWordIterator i = new JapaneseWordIterator( new File( "data/edict" ) ); i.hasNext(); )
			{
				word = (JapaneseWord) i.next();
				document = new Document();
				if( word.japaneseKanji != null ) document.add( Field.Text( "japaneseKanji", word.japaneseKanji ) );
				document.add( Field.Text( "japaneseKana", word.japaneseKana ) );
				document.add( Field.Text( "english", word.english ) );
				indexWriter.addDocument( document );
				if( (++count) % 100 == 0 )
				{
					System.out.print( "#" );
				}
			}
			System.out.println();
		}
		finally
		{
			if( indexWriter != null )
			{
				indexWriter.close();
			}
		}
	}
	
	public void indexKoreanWords( boolean create ) throws IOException
	{
		IndexWriter indexWriter = null;
		try
		{
			indexWriter = new IndexWriter( directory, analyzer, create );
			KoreanWord word;
			Document document;
			int count = 0;
			TermDocs docs;
			System.out.println( "Indexing engdic..." );
			for( KoreanWordIterator i = new KoreanWordIterator( new File( "data/engdic-v4" ) ); i.hasNext(); )
			{
				word = (KoreanWord) i.next();
				document = new Document();
				if( word.koreanHanja != null ) document.add( Field.Text( "koreanHanja", word.koreanHanja ) );
				document.add( Field.Text( "koreanHangul", word.koreanHangul ) );
				document.add( Field.Text( "english", word.english ) );
				indexWriter.addDocument( document );
				if( (++count) % 100 == 0 )
				{
					System.out.print( "#" );
				}
			}
			System.out.println();
		}
		finally
		{
			if( indexWriter != null )
			{
				indexWriter.close();
			}
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	// Private
	
	private static Analyzer analyzer = new ChineseAnalyzer();
	private Directory directory;
}
